/*

*/

import tree.*;
import tokenizer.*;
import treelanguage.*;
import language6.*;

/**
   This program parses and then evaluates strings from Language_6.
*/
public class Language_6_Examples
{
   public static void main(String[] args)
   {
      // IMPORTANT: Set this to 0 or 1 depending on whether you need
      // to see all of the interpreter's debugging information.
      EvaluateAST.DEBUG = 1;

      String[] programs =
      {
         // 0, apply an anonymous lambda expression
         "(apply (lambda x (* x x)) 7)",

         // 1,
         String.join("\n",
         "(prog",
           "(var g (lambda h (apply h 5)))",
           "(apply g (lambda x (* x x))))"), // pass an anonymous lambda as a parameter

         // 2,
         // Apply an anonymous function (with a function parameter)
         // to an anonymous function.
         "(apply (lambda h (apply h 7)) (lambda x (+ x x x)))",

         // 3,
         // Anonymous functions as return values.
         String.join("\n",
         "(prog",
         "  (var h (lambda b (if b (lambda x (* x x))      // anonymous lambda",
         "                         (lambda x (* x x x))))) // anonymous lambda",
         "  (var r1 (apply (apply h  true) 5))             // returns 25",
         "  (print r1)",
         "  (var r2 (apply (apply h false) 5)))           // returns 125"),

         // 4,
         // Another anonymous function as a return value.
         // (With dynamic scope, this program would return a different result.)
         String.join("\n",
         "(prog",
         "  (var x -100)",
         "  (var f (lambda x (lambda y (+ y x))))",
         "  (apply (apply f 5) 100))"),

         // 5,
         // Demonstrate a nested anonymous function parameter.
         // The final result should be 321.
         // (It would be 421 using dynamic scope.)
         String.join("\n",
         "(prog",
         "  (var w 300)",
         "  (var f1 (lambda x (apply f2 (lambda y (+ (+ x y) w)))))",
         "  (var f2 (lambda h (begin  // h is a function parameter",
         "                      (var w 400)",
         "                      (apply h 20)))) // call the function parameter",
         "  (apply f1 1))")
      };

      int i = 0;
      for (final String str : programs)
      {
         System.out.println("\n" + i + " =========================================");

         // Build and evaluate the abstract syntax tree that represents the program.
         try
         {
            final Tree<Token> ast = Parse2AST.parse( str );

            // Print the AST as an S-expression
            //System.out.println( EvaluateAST.toString(ast) + "\n" );

            // Pretty-print the abstract syntax tree.
            //System.out.println( PrettyPrintAST.prettyPrint( ast ) + "\n" );

            System.out.println("// Unparsed program: " );
            // Print the infix version of the expression.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            // Create dot and png files from the AST.
            final String baseName = String.format("Language_6_Examples.%02d", i);
            Tree2dot.tree2dot(ast, baseName);
            Tree2png.tree2png(baseName);

            // Create an xml file from the AST.
            AST2xml.ast2xml(ast, baseName);

            // Run the program (interpret the AST).
            final Value value = EvaluateAST.eval( ast );
            if (EvaluateAST.DEBUG > 0)
            {
               System.out.println("result = " + value + "\n" );
            }
            else
            {
               System.out.println("result = " + value.toSimpleString() + "\n" );
            }
         }
         catch (TokenizeError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (ParseError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (EvalError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         ++i;
      }
   }
}
