/*

*/

import tree.*;
import tokenizer.*;
import treelanguage.*;
import language6.*;

/**
   This program parses and then evaluates strings from Language_6.
   <p>
   The two examples in this program demonstrate the use of closures.
   <p>
   The two examples, makeCounter and makeAdder are ubiquitous examples
   that you can find implemented in every functional programming language.
*/
public class Language_6_Examples_2
{
   public static void main(String[] args)
   {
      // IMPORTANT: Set this to 0 or 1 depending on whether you need
      // to see all of the interpreter's debugging information.
      EvaluateAST.DEBUG = 1;

      String[] programs =
      {
         // 0,
         // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Closures#emulating_private_methods_with_closures
         String.join("\n",
         "(prog",
         "  (var makeCounter",
         "    (lambda x",
         "       (lambda (set x (+ x 1)))))",
         "  (var countFrom5  (apply makeCounter 5))",
         "  (var countFrom10 (apply makeCounter 10))",
         "  (var r1 (apply countFrom5))  // 6",
         "  (var r2 (apply countFrom5))  // 7",
         "  (var r3 (apply countFrom5))  // 8",
         "  (var r4 (apply countFrom10)) // 11",
         "  (var r5 (apply countFrom10)) // 12",
         "  (var r6 (apply countFrom5))) // 9"),

         // 1,
         // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Closures#closure
         String.join("\n",
         "(prog",
         "  (var makeAdder",
         "    (lambda x",
         "       (lambda y (+ x y))))",
         "  (var add5  (apply makeAdder  5))",
         "  (var add10 (apply makeAdder 10))",
         "  (var r1 (apply add5 2))   // 7",
         "  (var r2 (apply add10 2))) // 12")
      };

      int i = 0;
      for (final String str : programs)
      {
         System.out.println("\n" + i + " =========================================");

         // Build and evaluate the abstract syntax tree that represents the program.
         try
         {
            final Tree<Token> ast = Parse2AST.parse( str );

            // Print the AST as an S-expression
            //System.out.println( EvaluateAST.toString(ast) + "\n" );

            // Pretty-print the abstract syntax tree.
            //System.out.println( PrettyPrintAST.prettyPrint( ast ) + "\n" );

            System.out.println("// Unparsed program: " );
            // Print the infix version of the expression.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            // Create dot and png files from the AST.
            final String baseName = String.format("Language_6_Examples2.%02d", i);
            Tree2dot.tree2dot(ast, baseName);
            Tree2png.tree2png(baseName);

            // Create an xml file from the AST.
            AST2xml.ast2xml(ast, baseName);

            // Run the program (interpret the AST).
            final Value value = EvaluateAST.eval( ast );
            if (EvaluateAST.DEBUG > 0)
            {
               System.out.println("result = " + value + "\n" );
            }
            else
            {
               System.out.println("result = " + value.toSimpleString() + "\n" );
            }
         }
         catch (TokenizeError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (ParseError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (EvalError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         ++i;
      }
   }
}
