/*

*/

package language6;

import tree.Tree;
import tokenizer.Token;

/**
   This class implements "closure" objects.
   <p>
   The ip reference (instruction pointer) points to a block
   of code (the AST of a lambda expression) that may make
   references to non-local variables (so the block of code
   has "free variables").
   <p>
   The ep reference (environment pointer) points to an
   Environment object that "closes" the block of code by
   providing values (bindings) for the non-local references
   made by the block of code.
*/

public class IPEP
{
   public final Tree<Token> ip;   // the ip, the "instruction pointer"
   public final Environment ep;   // the ep, the "environment pointer"

   /**
      Construct an IPEP object (a closure).

      @param tree  the abstract syntax {@link Tree} node of the function's lambda expression
      @param env   the (@link Environment} that the lambda expression was defined in
   */
   public IPEP(Tree<Token> tree, Environment env)
   {
      this.ip = tree;
      this.ep = env;
   }


   @Override
   public String toString()
   {
      return EvaluateAST.toString(ip); // only the lambda part
   }
}
